//--------------------------------------------------------------
// File     : stm32_ub_terminal_adc.c
// Datum    : 11.09.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, ADC
// Funktion : ADC Funktionen fr das Terminal
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_terminal_adc.h"



//--------------------------------------------------------------
// globale Variablen
//--------------------------------------------------------------
ADC_CommonInitTypeDef ADC_CommonInitStructure; // muss global sein (k.A. warum)
ADC_InitTypeDef       ADC_InitStructure;  // muss global sein (k.A. warum)

static uint32_t adc_init_ok=0;
static uint16_t anz_mw=1,anz_bit=0;


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
uint16_t P_ADC_INT2PIN(uint32_t wert);
void P_initADC(void);


//--------------------------------------------------------------
// CMD : "ADC_SET_MW_x"
// Antwort : "CMD=OK"
//
// Einstellung der Mittelwertbildung
// x=Mode [0...7]
//--------------------------------------------------------------
void TERMINAL_ADC_SET_MW(char *ptr)
{
  uint32_t mode;

  mode=TERMINAL_STR2INT(ptr,11,1);
  if(ascii_err_flag!=0) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(mode==0) {
    anz_mw=1;anz_bit=0;
  }
  else if(mode==1) {
    anz_mw=2;anz_bit=1;
  }
  else if(mode==2) {
    anz_mw=4;anz_bit=2;
  }
  else if(mode==3) {
    anz_mw=8;anz_bit=3;
  }
  else if(mode==4) {
    anz_mw=16;anz_bit=4;
  }
  else if(mode==5) {
    anz_mw=32;anz_bit=5;
  }
  else if(mode==6) {
    anz_mw=64;anz_bit=6;
  }
  else if(mode==7) {
    anz_mw=128;anz_bit=7;
  }
  else {
    TERMINAL_SEND("CMD=?");
    return;
  }

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "ADC_IN_xyy"
// Antwort : "CMD=OK"
//
// GPIO Pin als Analog-Eingang definieren
// x=Port [A...F], yy=Pin [00 ...15]
// Hinweis : erlaubt sind nur die Pins : [A0-7, B0-1, C0-5]
//--------------------------------------------------------------
void TERMINAL_ADC_IN(char *ptr)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  uint32_t port;
  uint32_t pin;

  port=TERMINAL_STR2INT(ptr,7,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,8,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) {
    if(pin>7) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
  }
  else if(port==11) {
    if(pin>1) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
  }
  else if(port==12) {
    if(pin>5) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
  }
  else {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  if(port==11) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  if(port==12) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  if(port==13) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  if(port==14) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  if(port==15) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

  GPIO_InitStructure.GPIO_Pin = P_ADC_INT2PIN(pin);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;

  if(port==10) GPIO_Init(GPIOA, &GPIO_InitStructure);
  if(port==11) GPIO_Init(GPIOB, &GPIO_InitStructure);
  if(port==12) GPIO_Init(GPIOC, &GPIO_InitStructure);
  if(port==13) GPIO_Init(GPIOD, &GPIO_InitStructure);
  if(port==14) GPIO_Init(GPIOE, &GPIO_InitStructure);
  if(port==15) GPIO_Init(GPIOF, &GPIO_InitStructure);

  if(adc_init_ok==0) {
    P_initADC();
    adc_init_ok=1;
  }

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "ADC_RD_xyy"
// Antwort : "ADC_RD_xyy=0000" bis "ADC_RD_xyy=4095"
//
// Analogwert von ADC Pin auslesen
// x=Port [A...F], yy=Pin [00 ...15]
// Hinweis : erlaubt sind nur die Pins : [A0-7, B0-1, C0-5]
//--------------------------------------------------------------
void TERMINAL_ADC_READ(char *ptr)
{
  uint32_t port;
  uint32_t pin;
  uint8_t adc_ch;
  uint16_t messwert,n;
  uint32_t mittelwert=0; 

  port=TERMINAL_STR2INT(ptr,7,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,8,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) {
    if(pin>7) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
    else {
      if(pin==0) adc_ch=ADC_Channel_0;
      if(pin==1) adc_ch=ADC_Channel_1;
      if(pin==2) adc_ch=ADC_Channel_2;
      if(pin==3) adc_ch=ADC_Channel_3;
      if(pin==4) adc_ch=ADC_Channel_4;
      if(pin==5) adc_ch=ADC_Channel_5;
      if(pin==6) adc_ch=ADC_Channel_6;
      if(pin==7) adc_ch=ADC_Channel_7;
    }
  }
  else if(port==11) {
    if(pin>1) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
    else {
      if(pin==0) adc_ch=ADC_Channel_8;
      if(pin==1) adc_ch=ADC_Channel_9;
    }
  }
  else if(port==12) {
    if(pin>5) {
      TERMINAL_SEND("CMD=?");
      return;   
    }
    else {
      if(pin==0) adc_ch=ADC_Channel_10;
      if(pin==1) adc_ch=ADC_Channel_11;
      if(pin==2) adc_ch=ADC_Channel_12;
      if(pin==3) adc_ch=ADC_Channel_13;
      if(pin==4) adc_ch=ADC_Channel_14;
      if(pin==5) adc_ch=ADC_Channel_15;
    }
  }
  else {
    TERMINAL_SEND("CMD=?");
    return;
  }

  // Messkanal einrichten
  ADC_RegularChannelConfig(ADC1,adc_ch,1,ADC_SampleTime_3Cycles);

  mittelwert=0;
  for(n=0;n<anz_mw;n++) {
    // Messung starten
    ADC_SoftwareStartConv(ADC1);
    // warte bis Messung fertig ist
    while(ADC_GetFlagStatus(ADC1,ADC_FLAG_EOC)==RESET);
    // Messwert auslesen
    messwert=ADC_GetConversionValue(ADC1);
    // Mittelwert bilden    
    mittelwert+=messwert;
  }

  messwert=(mittelwert >> anz_bit);


  TERMINAL_SEND_INT(ptr,messwert,1);
}


//--------------------------------------------------------------
// Integer Nr. in Port-Pin umwandeln
//--------------------------------------------------------------
uint16_t P_ADC_INT2PIN(uint32_t wert)
{
  uint16_t ret_wert=0;

  if(wert==0) ret_wert=GPIO_Pin_0;
  if(wert==1) ret_wert=GPIO_Pin_1;
  if(wert==2) ret_wert=GPIO_Pin_2;
  if(wert==3) ret_wert=GPIO_Pin_3;
  if(wert==4) ret_wert=GPIO_Pin_4;
  if(wert==5) ret_wert=GPIO_Pin_5;
  if(wert==6) ret_wert=GPIO_Pin_6;
  if(wert==7) ret_wert=GPIO_Pin_7;
  if(wert==8) ret_wert=GPIO_Pin_8;
  if(wert==9) ret_wert=GPIO_Pin_9;
  if(wert==10) ret_wert=GPIO_Pin_10;
  if(wert==11) ret_wert=GPIO_Pin_11;
  if(wert==12) ret_wert=GPIO_Pin_12;
  if(wert==13) ret_wert=GPIO_Pin_13;
  if(wert==14) ret_wert=GPIO_Pin_14;
  if(wert==15) ret_wert=GPIO_Pin_15;

  return(ret_wert);
}


//--------------------------------------------------------------
// init vom ADC
//--------------------------------------------------------------
void P_initADC(void)
{
  // Clock Enable
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

  // ADC-Config
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC1s_VORTEILER;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);

  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;
  ADC_Init(ADC1, &ADC_InitStructure);

  // ADC-Enable
  ADC_Cmd(ADC1, ENABLE);

}
