//--------------------------------------------------------------
// File     : stm32_ub_terminal_gpio.c
// Datum    : 11.09.2013
// Version  : 1.2
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO
// Funktion : GPIO Funktionen fr das Terminal
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_terminal_gpio.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
uint16_t P_INT2PIN(uint32_t wert);


//--------------------------------------------------------------
// CMD : "GPIO_OUT_xyy"
// Antwort : "CMD=OK"
//
// GPIO Pin als Ausgang definieren
// x=Port [A...F], yy=Pin [00 ...15]
//--------------------------------------------------------------
void TERMINAL_GPIO_OUT(char *ptr)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  uint32_t port;
  uint32_t pin;

  port=TERMINAL_STR2INT(ptr,9,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,10,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  if(port==11) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  if(port==12) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  if(port==13) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  if(port==14) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  if(port==15) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

  GPIO_InitStructure.GPIO_Pin = P_INT2PIN(pin);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  if(port==10) GPIO_Init(GPIOA, &GPIO_InitStructure);
  if(port==11) GPIO_Init(GPIOB, &GPIO_InitStructure);
  if(port==12) GPIO_Init(GPIOC, &GPIO_InitStructure);
  if(port==13) GPIO_Init(GPIOD, &GPIO_InitStructure);
  if(port==14) GPIO_Init(GPIOE, &GPIO_InitStructure);
  if(port==15) GPIO_Init(GPIOF, &GPIO_InitStructure);

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "GPIO_LO_xyy"
// Antwort : "CMD=OK"
//
// GPIO Pin auf Lo-Pegel setzen
// x=Port [A...F], yy=Pin [00 ...15]
//--------------------------------------------------------------
void TERMINAL_GPIO_LO(char *ptr)
{
  uint32_t port;
  uint32_t pin;

  port=TERMINAL_STR2INT(ptr,8,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,9,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) GPIOA->BSRRH = P_INT2PIN(pin);
  if(port==11) GPIOB->BSRRH = P_INT2PIN(pin);
  if(port==12) GPIOC->BSRRH = P_INT2PIN(pin);
  if(port==13) GPIOD->BSRRH = P_INT2PIN(pin);
  if(port==14) GPIOE->BSRRH = P_INT2PIN(pin);
  if(port==15) GPIOF->BSRRH = P_INT2PIN(pin);

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "GPIO_HI_xyy"
// Antwort : "CMD=OK"
//
// GPIO Pin auf Hi-Pegel setzen
//  x=Port [A...F], yy=Pin [00 ...15]
//--------------------------------------------------------------
void TERMINAL_GPIO_HI(char *ptr)
{
  uint32_t port;
  uint32_t pin;

  port=TERMINAL_STR2INT(ptr,8,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,9,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) GPIOA->BSRRL = P_INT2PIN(pin);
  if(port==11) GPIOB->BSRRL = P_INT2PIN(pin);
  if(port==12) GPIOC->BSRRL = P_INT2PIN(pin);
  if(port==13) GPIOD->BSRRL = P_INT2PIN(pin);
  if(port==14) GPIOE->BSRRL = P_INT2PIN(pin);
  if(port==15) GPIOF->BSRRL = P_INT2PIN(pin);

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "GPIO_IN_mm_xyy"
// Antwort : "CMD=OK"
//
// GPIO Pin als Eingang definieren
// mm=Widerstand [PU, PD, WO]
// x=Port [A...F], yy=Pin [00 ...15]
//--------------------------------------------------------------
void TERMINAL_GPIO_IN(char *ptr, uint8_t mode)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  uint32_t port;
  uint32_t pin;

  port=TERMINAL_STR2INT(ptr,11,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,12,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  if(port==11) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  if(port==12) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  if(port==13) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  if(port==14) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  if(port==15) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

  GPIO_InitStructure.GPIO_Pin = P_INT2PIN(pin);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  if(mode==0) GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  if(mode==1) GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  if(mode==2) GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  if(port==10) GPIO_Init(GPIOA, &GPIO_InitStructure);
  if(port==11) GPIO_Init(GPIOB, &GPIO_InitStructure);
  if(port==12) GPIO_Init(GPIOC, &GPIO_InitStructure);
  if(port==13) GPIO_Init(GPIOD, &GPIO_InitStructure);
  if(port==14) GPIO_Init(GPIOE, &GPIO_InitStructure);
  if(port==15) GPIO_Init(GPIOF, &GPIO_InitStructure);

  TERMINAL_SEND("CMD=OK");
}


//--------------------------------------------------------------
// CMD : "GPIO_RD_xyy"
// Antwort : "GPIO_RD_xyy=0" oder "GPIO_RD_xyy=1"
//
// Pegel von GPIO Pin auslesen
//  x=Port [A...F], yy=Pin [00 ...15]
//--------------------------------------------------------------
void TERMINAL_GPIO_READ(char *ptr)
{
  uint32_t port;
  uint32_t pin;
  BitAction wert;

  port=TERMINAL_STR2INT(ptr,8,1);
  if((ascii_err_flag!=0) || (port<10) || (port>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  pin=TERMINAL_STR2INT(ptr,9,2);
  if((ascii_err_flag!=0) || (pin>15)) {
    TERMINAL_SEND("CMD=?");
    return;
  }

  if(port==10) wert=GPIO_ReadInputDataBit(GPIOA, P_INT2PIN(pin));
  if(port==11) wert=GPIO_ReadInputDataBit(GPIOB, P_INT2PIN(pin));
  if(port==12) wert=GPIO_ReadInputDataBit(GPIOC, P_INT2PIN(pin));
  if(port==13) wert=GPIO_ReadInputDataBit(GPIOD, P_INT2PIN(pin));
  if(port==14) wert=GPIO_ReadInputDataBit(GPIOE, P_INT2PIN(pin));
  if(port==15) wert=GPIO_ReadInputDataBit(GPIOF, P_INT2PIN(pin));

  if(wert==Bit_RESET) {
    TERMINAL_SEND_INT(ptr,0,0);    
  }
  else {
    TERMINAL_SEND_INT(ptr,1,0);
  }
}


//--------------------------------------------------------------
// Integer Nr. in Port-Pin umwandeln
//--------------------------------------------------------------
uint16_t P_INT2PIN(uint32_t wert)
{
  uint16_t ret_wert=0;

  if(wert==0) ret_wert=GPIO_Pin_0;
  if(wert==1) ret_wert=GPIO_Pin_1;
  if(wert==2) ret_wert=GPIO_Pin_2;
  if(wert==3) ret_wert=GPIO_Pin_3;
  if(wert==4) ret_wert=GPIO_Pin_4;
  if(wert==5) ret_wert=GPIO_Pin_5;
  if(wert==6) ret_wert=GPIO_Pin_6;
  if(wert==7) ret_wert=GPIO_Pin_7;
  if(wert==8) ret_wert=GPIO_Pin_8;
  if(wert==9) ret_wert=GPIO_Pin_9;
  if(wert==10) ret_wert=GPIO_Pin_10;
  if(wert==11) ret_wert=GPIO_Pin_11;
  if(wert==12) ret_wert=GPIO_Pin_12;
  if(wert==13) ret_wert=GPIO_Pin_13;
  if(wert==14) ret_wert=GPIO_Pin_14;
  if(wert==15) ret_wert=GPIO_Pin_15;

  return(ret_wert);
}
